#!/usr/local/bin/python  
# PyTree launcher script 
# wrappers for viewing tree types in the book, plus test cases/gui 
 
import string 
from Tkinter import * 
from treeview_wrappers import TreeWrapper, TreeViewer 
import parser2 
 

 

 
################################################################### 
# parse tree wrapper 
################################################################### 
 
class ParseTreeWrapper(TreeWrapper): 
    def __init__(self):                        # embed parse tree in viewer 
        self.dict = {}                         # adds viewer protocols 
    def children(self, node): 
        try: 
            return [node.left, node.right] 
        except: 
            try: 
                return [node.var, node.val] 
            except:  
                return None 
    def label(self, node): 
        for attr in ['label', 'num', 'name']: 
            if hasattr(node, attr): 
                return str(getattr(node, attr)) 
        return 'set' 
    def onClick(self, node):                        # on tree label click 
        try:                                        # tree-specific action 
            result = node.apply(self.dict)          # evaluate subtree 
            return 'Value = ' + str(result)         # show result in popup 
        except: 
            return 'Value = <error>' 
    def onInputLine(self, line, viewer):            # on input line 
        p = parser2.Parser()                        # parse expr text 
        p.lex.newtext(line)                         # draw resulting tree 
        t = p.analyse() 
        if t: viewer.drawTree(t) 
 
################################################################### 
# canned test cases (or type new nodelists/exprs in input field) 
################################################################### 
 
def shownodes(sequence): 
    sequence = map(str, sequence)           # convert nodes to strings 
    entry.delete(0, END)                    # show nodes in text field 
    entry.insert(0, string.join(sequence, ' ')) 
 

 
def test_parser(expr): 
    parser = parser2.Parser()               # tree type is parser wrapper 
    parser.lex.newtext(expr)                # subtrees evaluate when clicked 
    tree   = parser.analyse()               # input line parses new expr 
    entry.delete(0, END)                    # vars set in wrapper dictionary 
    entry.insert(0, expr)                   # see lang/text chapter for parser 
    if tree: viewer.drawTree(tree) 
 
def test1_parser(): test_parser("1 + 3 * (2 * 3 + 4)")     
def test2_parser(): test_parser("set temp 1 + 3 * 2 * 3 + 4") 
def test3_parser(): test_parser("set result temp + ((1 + 3) * 2) * (3 + 4)") 
 
################################################################### 
# build viewer with extra widgets to test tree types 
################################################################### 
 
if __name__ == '__main__': 
    root = Tk()                             # build a single viewer gui 
    pwrapper = ParseTreeWrapper()           # make wrapper objects 
    viewer   = TreeViewer(pwrapper, root)   # start out in binary mode 
 
    def onRadio(): 
            viewer.setTreeType(pwrapper) 
            for btn in p_btns: btn.pack(side=LEFT) 
 
    var = StringVar() 
    var.set('ptree') 
    
    Radiobutton(root, text='Parser', command=onRadio,  
                      variable=var, value='ptree').pack(side=LEFT) 
 
    p_btns = [] 
    p_btns.append(Button(root, text='test1', command=test1_parser)) 
    p_btns.append(Button(root, text='test2', command=test2_parser)) 
    p_btns.append(Button(root, text='test3', command=test3_parser)) 
    onRadio() 
 
    def onInputLine(): 
        line = entry.get()              # use per current tree wrapper type 
        viewer.onInputLine(line)        # type a node list or expression 
 
    Button(root, text='input', command=onInputLine).pack(side=RIGHT) 
    entry = Entry(root) 
    entry.pack(side=RIGHT, expand=YES, fill=X) 
    entry.bind('<Return>', lambda event: onInputLine())   # button or enter key 
    root.mainloop()                                       # start up the gui 
